#include "StdAfx.h"
#include "ChatModel.h"
#include "resource.h"
#include "RecentMessageSqlite.h"

CChatModel::CChatModel(void)
{
	m_nRoomID = 0;
	m_nChatRoomSequecenNumber = 0;
	m_bGroupRoom = FALSE;
	m_strInviteUserName = _T("");	
	//ZeroMemory(&m_stMyData, sizeof(GT_MY_USER_DATA));
	RemoveAllChatMessage();
	ResetMyMessageTid();
	ResetMyTranslateTid();
	ResetMyTranslateFileInfo();
	::ZeroMemory(&m_cfDefaultRecvChatView, sizeof(CHARFORMAT2));    //lint !e1415
	::ZeroMemory(&m_cfUserSendChatView, sizeof(CHARFORMAT2));       //lint !e1415
	m_strRoomType = _T("new");
	m_nMsgTid = 1;
	m_nChatSrvPort = -1;

	m_eSessionCreate	= SESSION_UNINITIALIZED;
	m_eInitRoom			= INITROON_UNINITIALIZED;
	m_eInviteList		= REGISTER_UNINITIALIZED;

	m_eSessionReason	= REASON_NONE;
	m_nCreateSessionCheckTimes = 0;
	m_bHaveMsgRecord = FALSE;
    m_eChatMakeType = MAKETYPE_CREATE;
}

CChatModel::~CChatModel(void)
{
    CSingleLock lock(&m_csMapUserList, TRUE);
	if(m_mapUserList.empty() == false)
	{		
		map<CString, CChatUser*>::iterator user_iter = m_mapUserList.begin();
		for(; user_iter != m_mapUserList.end(); ++user_iter)
		{
			CChatUser* pUser = (CChatUser*)user_iter->second;
			if(pUser)
			{
				TRACE("%s%d%x\n", __FUNCTION__, __LINE__, pUser);
				delete pUser;
				pUser = NULL;
			}
		}

		m_mapUserList.clear();	
	}	

	RemoveAllChatMessage();
	ResetMyMessageTid();
	ResetMyTranslateTid();
	ResetMyTranslateFileInfo();
}

void CChatModel::SetChatInfo(int nSeqNumber, const vector<CChatUser*>& userList, EChatMakeType enumMakeType)
{
	m_eChatMakeType = enumMakeType;
	m_nChatRoomSequecenNumber = nSeqNumber;
	SetChatUsers(userList);		
}

void CChatModel::SetChatUsers(const vector<CChatUser*>& userList)
{	
    CSingleLock lock(&m_csMapUserList, TRUE);
	vector<CChatUser*>::const_iterator user_iter = userList.begin();
	
	for (; user_iter != userList.end(); ++user_iter)
	{
		CChatUser* pUser = (CChatUser*)(*user_iter);

		if (!pUser)
			continue;

		if (GetMyID().CompareNoCase(pUser->GetID()) != 0)
		{
			m_mapUserList.insert(make_pair(pUser->GetID(), pUser));
		}
	}
    SortChatUserVector();
	if (GetChatUserMapCount() > 1)
		SetGroupRoom(TRUE);
}

void CChatModel::SetServerInfo(LPCTSTR strSrvAddress, int nSrvPort, int nRoomID, LPCTSTR strRoomType)
{
	m_strChatSrvAddress = strSrvAddress;
	m_nChatSrvPort = nSrvPort;
	m_nRoomID = nRoomID;	
	SetRoomType(strRoomType);
}

bool CChatModel::IsValidServerInfo()
{
	//  ּ, Ʈ, ȣ ϳ      .
	if (m_strChatSrvAddress.IsEmpty() || m_nChatSrvPort == -1 || m_nRoomID == 0)
		return false;

	return true;
}

void CChatModel::SetMyUserData(GT_MY_USER_DATA* pstMyUserData)
{
	memcpy(&m_stMyData, pstMyUserData, sizeof(GT_MY_USER_DATA));
}

void CChatModel::SetRecvChatViewDefaultFont(CHARFORMAT2* pcfDefault)
{
	memcpy(&m_cfDefaultRecvChatView, pcfDefault, sizeof(CHARFORMAT2));  //lint !e123

// 	m_cfDefaultRecvChatView.cbSize = sizeof(CHARFORMAT2);
// 	m_cfDefaultRecvChatView.dwMask = CFM_COLOR;
// 	m_cfDefaultRecvChatView.dwEffects = 0; // To disable CFE_AUTOCOLOR	
// 	m_cfDefaultRecvChatView.yHeight = 10 * 20;
// 	m_cfDefaultRecvChatView.crTextColor = RGB(0, 0, 0);
}

void CChatModel::SetSendChatViewUserFont(CHARFORMAT2* pcfUser)
{
	memcpy(&m_cfUserSendChatView, pcfUser, sizeof(CHARFORMAT2));    //lint !e123	
}

void   CChatModel::SetTrayPopupInfo(LPCTSTR strID, LPCTSTR strNickName, LPCTSTR strMsg , int nMsgType){
	
	m_trayPopInfo.strID.Format(_T("%s"), strID);
	if (nMsgType == DEF_CHAT_MSG_TYPE_CHAT)
	{
		if(wcslen(strNickName) > 0)
			m_trayPopInfo.strNickName.Format(_T("%s"), strNickName);		
		m_trayPopInfo.strMsg.Format(_T("%s"), strMsg);
	}
	else if (nMsgType == DEF_CHAT_MSG_TYPE_STICKER)
	{
		if(wcslen(strNickName) > 0)
			m_trayPopInfo.strNickName.Format(_T("%s"), strNickName);

		m_trayPopInfo.strMsg.Format(_T("(%s)"), LoadStringFromResource(IDS_STRING377));
	}
	else if(nMsgType == DEF_FTP_MSG_TYPE)
	{
		if (wcslen(strNickName) > 0)
		{
			m_trayPopInfo.strNickName.Format(TEXT("%s"), strNickName);
		}
		m_trayPopInfo.strMsg.Format(_T("%s"), /*LoadStringFromResource(IDS_CHAT_RECV_FILE_POPUP),*/ strMsg);
	}
}



void CChatModel::SetPopupMessage(LPCTSTR strNickName, LPCTSTR strMsg, int nMsgType)
{
	CString strPopUpMessageFormat;
	strPopUpMessageFormat.LoadString(IDS_CHAT_POPUPMESSAGE);
	if (nMsgType == DEF_CHAT_MSG_TYPE_CHAT)
	{
		if(wcslen(strNickName) > 0)
			m_strPopupMessage.Format(_T("%s"), strNickName);		
		m_strPopupMessage.Format(_T("%s"), strMsg);
	}
	else if (nMsgType == DEF_CHAT_MSG_TYPE_STICKER)
	{
		if(wcslen(strNickName) > 0)
			m_strPopupMessage.Format(_T("%s"), strNickName);
		
		m_strPopupMessage.Format(_T("(%s)"), LoadStringFromResource(IDS_STRING377));
	}
	else if(nMsgType == DEF_FTP_MSG_TYPE)
	{
		if (wcslen(strNickName) > 0)
		{
			m_strPopupMessage.Format(TEXT("%s"), strNickName);
		}
		
	   m_strPopupMessage.Format(_T("%s\n\n%s"), LoadStringFromResource(IDS_CHAT_RECV_FILE_POPUP), strMsg);
		
	}
}


void CChatModel::InitInviteUserStatus()
{
	CSingleLock lock(&m_csMapUserList, TRUE);
	//lock.Lock();

	map<CString, CChatUser*>::iterator user_iter = m_mapUserList.begin();
	for(; user_iter != m_mapUserList.end(); ++user_iter)
	{
		CChatUser* pUser = user_iter->second;
		pUser->SetInviteStatus(FALSE);
	}

	//lock.Unlock();
}

void CChatModel::SetInviteUserStatus(LPCTSTR strID, BOOL bInviteStatus)
{
	CSingleLock lock(&m_csMapUserList, TRUE);
	//lock.Lock();
	map<CString, CChatUser*>::iterator user_iter = m_mapUserList.find(strID);
	if(user_iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = user_iter->second;
        pChatUser->SetInviteStatus(bInviteStatus);

	}	
	//lock.Unlock();
}

void CChatModel::SetInviteUserStatus(vector<LPCTSTR> vecID, BOOL bInviteStatus)
{
	vector<LPCTSTR>::iterator iter = vecID.begin();
	for(; iter != vecID.end(); iter++)
	{
		SetInviteUserStatus((*iter), bInviteStatus);
	}
}

void CChatModel::SetTalkNameList(vector<NT_TALKNAME_DATA> vecTalkName)
{
    {
        CSingleLock lock(&m_csTalkNameList, TRUE);
        vector<NT_TALKNAME_DATA>::const_iterator talkname_iter = vecTalkName.begin();
        for(; talkname_iter != vecTalkName.end(); ++talkname_iter)
        {
            m_mapTalkNameList.insert(make_pair(talkname_iter->strUserID.c_str(), talkname_iter->strUserNick.c_str()));
        }
    }
	RefreshTalkName();
}

BOOL CChatModel::ChangeNickName(LPCTSTR strID, LPCTSTR strNickName)
{	
	BOOL bRet = FALSE;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.find(strID);
	if(user_iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = user_iter->second;
		if(pChatUser->GetNickName().Compare(strNickName) != 0)
		{
			pChatUser->SetNickName(strNickName);
			bRet = TRUE;
		}
	}

	return bRet;
}

BOOL CChatModel::SetChatUserInfo(const NE_MEMBER_DATA* pUserData)
{
	if (NULL == pUserData)
		return FALSE;

    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.find(pUserData->strUserId.c_str());

	if (user_iter == m_mapUserList.end())
		return FALSE;

	CChatUser* pChatUser = user_iter->second;

	if (!pChatUser)
		return FALSE;

	
	CString strName = pUserData->strName.c_str();
	if (strName.IsEmpty())
	{
		switch (GET_CONFIG()->GetLanguage())
		{
		case 0:
			strName = pUserData->strNameKR.c_str();
			break;
		case 1:
			strName = pUserData->strNameUS.c_str();
			break;
		case 2:
			strName = pUserData->strNameJP.c_str();
			break;
		case 3:
			strName = pUserData->strNameTW.c_str();
			break;
		case 4:
			strName = pUserData->strNameCN.c_str();
			break;
		default:
			strName = pUserData->strNameKR.c_str();
			break;
		}
	}
	
	pChatUser->SetName(strName);	// ActiveX ȭ ʴ  ̸ Ƿ ⼭ äش.

	if (pChatUser->GetNickName().IsEmpty())
		pChatUser->SetNickName(pUserData->strNickName.c_str());

	if (pChatUser->GetDeptName().IsEmpty())
		pChatUser->SetDeptNm(pUserData->strDept.c_str());

	if (pChatUser->GetExtensionTel().IsEmpty())
		pChatUser->SetExtensionTel(pUserData->strExtesion.c_str());

	if (pChatUser->GetCellphone().IsEmpty())
		pChatUser->SetCellphone(pUserData->strCellPhone.c_str());

	if (pChatUser->GetJobPos().IsEmpty())
		pChatUser->SetJobPos(pUserData->strJobPos.c_str());

	if (pChatUser->GetJobDuty().IsEmpty())
		pChatUser->SetJobDuty(pUserData->strJobDuty.c_str());

	pChatUser->SetEmail(pUserData->strEmail.c_str());
	pChatUser->SetUserInfoLoad(TRUE);

	return TRUE;
}

void CChatModel::AddChatUser(CChatUser* pUser)
{
    {
	    CSingleLock lock(&m_csMapUserList, TRUE);
	    //lock.Lock();
	    if (GetMyID().CompareNoCase(pUser->GetID()) != 0)
		    m_mapUserList.insert(make_pair(pUser->GetID(), pUser));	
	    //lock.Unlock();
    }
	if (GetChatUserMapCount() > 1)
		SetGroupRoom(TRUE);
}

void CChatModel::AddChatUser(LPCTSTR strUserID, LPCTSTR strUserName, LPCTSTR strUserNickName, GTUserStatusType status, BOOL bInviteStatus)
{
	CChatUser* pChatUser = new CChatUser();
	if (NULL != pChatUser)
	{
		pChatUser->SetID(strUserID);
		pChatUser->SetName(strUserName);
		pChatUser->SetNickName(strUserNickName);
		pChatUser->SetStatus(status);
		pChatUser->SetInviteStatus(bInviteStatus);

		AddChatUser(pChatUser);
	}
}


int CChatModel::FindUserMap(LPCTSTR strUserID)
{
	int nIdx = 0;	

    CSingleLock lock(&m_csMapUserList, TRUE);
	if(m_mapUserList.empty() == true)
		return USERLIST_EMPTY;

	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.begin();

	for(; user_iter != m_mapUserList.end(); ++user_iter)
	{
		CString strID = (CString)(user_iter->first);
		if(strID.CompareNoCase(strUserID) == 0)
			return nIdx;
		nIdx++;
	}

	return USERLIST_NOTFOUND;	
}

void CChatModel::AddChatMessage(LPCTSTR strID, LPCTSTR strName, LPCTSTR strNickName, int time, LPCTSTR strMsg, CHARFORMAT2* pFont, UINT nMsgSeq, UINT nTid, BYTE msgType)
{
/*	 ޽ ߺ ϱ  ID + tid  üũϰ ־, msg_seq  ϴ  .
	̷л msg_seq  Ȯϴ   Ȯ ̳, Ȥ  ߻Ѵٸ    ʿ . 2012.07.18. 
	if (IsAlreadyExistChatTid(strID, nTid) == TRUE)
		return;
	AddChatMessageTid(strID, nTid);
*/

	if (IsAlreadyExistMsgSequence(nMsgSeq))
		return;

	AddMsgSequence(nMsgSeq);

	CHAT_MSG* pNew = new CHAT_MSG;
	if (NULL != pNew)
	{
		pNew->strID.Format(_T("%s"), strID);
		pNew->strName.Format(_T("%s"), strName);
		pNew->strNickName.Format(_T("%s"), strNickName);
		pNew->pszMsg.Format(_T("%s"), strMsg);
		pNew->time = time;
		pNew->nMsgSeq = nMsgSeq;
		pNew->nTid = nTid;
		pNew->msgType = msgType;

		if (pFont)
		{
			pNew->cf = *pFont;
		}
		else
		{
			pNew->cf = m_cfDefaultRecvChatView;
		}
	}

	AddChatMessage(pNew);

	if (NULL != pFont)
		delete pFont;
}

void CChatModel::AddMsgSequence(UINT nMsgSeq)
{
	m_listMsgSeq.push_back(nMsgSeq);
}

bool CChatModel::IsAlreadyExistMsgSequence(UINT nMsgSeq)
{
	list<UINT>::iterator msg_iter = std::find(m_listMsgSeq.begin(), m_listMsgSeq.end(), nMsgSeq);

	if (msg_iter != m_listMsgSeq.end())
		return true;

	return false;
}

BOOL CChatModel::IsAlreadyExistChatTid(LPCTSTR strID, UINT nTid)
{
	map< CString, list<UINT> >::iterator user_iter = m_mapChatUserTid.find(strID);
	
	if (user_iter != m_mapChatUserTid.end())
	{
		list<UINT> listTid = user_iter->second;
		list<UINT>::iterator tid_iter = std::find(listTid.begin(), listTid.end(), nTid);
		if (tid_iter != listTid.end())
		{
			return TRUE;
		}
	}	
	return FALSE;
}

void CChatModel::AddChatMessageTid(LPCTSTR strID, UINT nTid)
{
	map< CString, list<UINT> >::iterator user_iter = m_mapChatUserTid.find(strID);

	if (user_iter != m_mapChatUserTid.end())
	{
		list<UINT>& listTid = user_iter->second;	// listTid     push ص   ݿ ʴ´.
		listTid.push_back(nTid);
	}
	else
	{
		list<UINT> listTid;
		listTid.push_back(nTid);
		m_mapChatUserTid.insert(make_pair(strID, listTid));
	}
}

void CChatModel::ResetTid(LPCTSTR strID)
{
	map< CString, list<UINT> >::iterator user_iter = m_mapChatUserTid.find(strID);
	if (user_iter != m_mapChatUserTid.end())
	{
		list<UINT> listTid = user_iter->second;
		listTid.clear();
		m_mapChatUserTid.erase(user_iter);
	}
}

trid_t CChatModel::GetChatTrid()
{
	trid_t trid(CHAT_BASE_TRID + m_nChatRoomSequecenNumber);
	return trid;
}

CString CChatModel::GetChatUserID(int idx)
{
    CSingleLock lock(&m_csMapUserList, TRUE);
	if(m_mapUserList.empty() == true)
		return _T("");
	map<CString, CChatUser*>::iterator iter = m_mapUserList.begin();

	if(idx == 0)
	{
		return iter->first;
	}
	for(int i = 0; i < idx; i++)
	{
		iter++;
	}

	return iter->first;
}


int __cdecl CChatModel::ItemComareFunc(void *pvlocale,const void *elem1, const void *elem2)
{
	if (!pvlocale || !elem1 || !elem2)
		return 0;

	/*enum GTUserStatusType
	{
		ustOFFLINE		= 0x00,				// 
		ustONLINE		= 0x01,				// ¶
		ustAWAY			= 0x02,				// ڸ
		ustBUSY			= 0x03,				// ٸ 빫 
		ustOUTSIDE		= 0x04,				// ܱ 
		ustEAT			= 0x05,				// Ļ 
		ustMEETING		= 0x06,				// ȸ 
		ustAUTOAWAY		= 0x07,				// ڵ ڸ
		ustMOBILE		= 0x10,				//  ¶
		ustUnknown		= 0x99,				//    
	};*/
	CChatUser **pUser1 = (CChatUser**)elem1;
	CChatUser **pUser2 = (CChatUser**)elem2;

	int Sts1 = (int)(*pUser1)->GetStatus();
	int Sts2 = (int)(*pUser2)->GetStatus();

	CString Name1 = (*pUser1)->GetName();
	CString Name2 = (*pUser2)->GetName();

	return BuddyItemComareFunc(STRUCT_BUDDY_COMPARE(Name1,Sts1), STRUCT_BUDDY_COMPARE(Name2,Sts2));
	
}

void CChatModel::SortChatUserVector()
{
	CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.begin();

	m_vectorChatUsr.clear();
	for(; user_iter != m_mapUserList.end(); ++user_iter)
	{
		CChatUser *pUser = user_iter->second;
		m_vectorChatUsr.push_back(pUser);
	}

	if(m_vectorChatUsr.size() > 1)
	{
		qsort_s(&(m_vectorChatUsr.at(0)), m_vectorChatUsr.size(), sizeof(CChatUser*), CChatModel::ItemComareFunc, this);	
	}
}

vector<CChatUser*> CChatModel::GetSortedChatUserVector()
{
	return m_vectorChatUsr; 
}

void CChatModel::GetChatUserArray(std::vector<CChatUser*> &chatUsrArray){

    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.begin();

	for(; user_iter != m_mapUserList.end(); ++user_iter)
	{
		CChatUser *pUser = user_iter->second;
	    chatUsrArray.push_back(pUser);

		
	}

}


CChatUser* CChatModel::GetChatUsrPtr(LPCTSTR strID)
{
	CChatUser* pChatUser = NULL;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		pChatUser = iter->second;
	}
	return pChatUser;
}


CString CChatModel::GetChatUserName(LPCTSTR strID)
{
	CString strUserName;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		strUserName = pChatUser->GetName();
	}
	else
	{
		strUserName = _T("");
	}

	return strUserName;
}

CString CChatModel::GetChatUserExtensionTel(LPCTSTR strID)
{
	CString strUserName;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		strUserName = pChatUser->GetExtensionTel();
	}
	else
	{
		strUserName = _T("");
	}

	return strUserName;
}

CString CChatModel::GetChatUserCellphone(LPCTSTR strID)
{
	CString strUserName;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		strUserName = pChatUser->GetCellphone();
	}
	else
	{
		strUserName = _T("");
	}

	return strUserName;
}

CString CChatModel::GetChatUserJobPos(LPCTSTR strID)
{
	CString strUserName;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		strUserName = pChatUser->GetJobPos();
	}
	else
	{
		strUserName = _T("");
	}

	return strUserName;
}

CString CChatModel::GetChatUserJobDuty(LPCTSTR strID)
{
	CString strUserName;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		strUserName = pChatUser->GetJobDuty();
	}
	else
	{
		strUserName = _T("");
	}

	return strUserName;
}

CString CChatModel::GetChatUserDeptName(LPCTSTR strID)
{
	CString strDeptName;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		strDeptName = pChatUser->GetDeptName();
	}
	else
	{
		strDeptName = _T("");
	}

	return strDeptName;
}

void CChatModel::SetAllChatUserOffline()
{
	 CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.begin();
	for(; iter != m_mapUserList.end(); iter++)
	{
		CChatUser* pChatUser = iter->second;
		pChatUser->SetStatus(ustOFFLINE);
	}

}

void CChatModel::SetChatUserStatus(LPCTSTR strID, GTUserStatusType status)
{
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		pChatUser->SetStatus(status);
	}
}

void CChatModel::SetChatUserIptStatus(LPCTSTR strID, NTIPTStatusType iptStatus)
{
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		pChatUser->SetIptStatus(iptStatus);
	}
}

CString CChatModel::GetOfflineUser()
{
	CString resultString(_T(""));
    CSingleLock lock(&m_csMapUserList, TRUE);
	if(m_mapUserList.empty() == true)
		return resultString;

	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.begin();

	bool bFirst = true;
	for(; user_iter != m_mapUserList.end(); ++user_iter)
	{
		CChatUser *pUser = user_iter->second;
		GTUserStatusType status = pUser->GetStatus();

		if (ustOFFLINE == status)
		{
			if (bFirst)
			{
				bFirst = false;
			}
			else
			{
				resultString += _T(", ");
			}

			resultString += pUser->GetName();
		}
	}

	return resultString;
}

int CChatModel::GetOfflineUserCount()
{
	int nOfflineUserCount = 0;

    CSingleLock lock(&m_csMapUserList, TRUE);
	if (m_mapUserList.empty() == true)
		return nOfflineUserCount;

	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.begin();

	//bool bFirst = true;

	for (; user_iter != m_mapUserList.end(); ++user_iter)
	{
		CChatUser *pUser = user_iter->second;
		GTUserStatusType status = pUser->GetStatus();

		if (ustOFFLINE == status)
			nOfflineUserCount++;
	}

	return nOfflineUserCount;
}

int CChatModel::GetOnlineUserCount()
{
	int nOnlineUserCount = 0;
    CSingleLock lock(&m_csMapUserList, TRUE);

	if (m_mapUserList.empty() == true)
		return nOnlineUserCount;

	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.begin();

	bool bFirst = true;
	
	for (; user_iter != m_mapUserList.end(); ++user_iter)
	{
		CChatUser *pUser = user_iter->second;
		GTUserStatusType status = pUser->GetStatus();

		if (ustOFFLINE != status)
			nOnlineUserCount++;
	}

	return nOnlineUserCount;
}

int CChatModel::GetUserCount()
{
    CSingleLock lock(&m_csMapUserList, TRUE);
	return m_mapUserList.size();
}

CString CChatModel::GetAllUserId()
{
	CString resultString(_T(""));

    CSingleLock lock(&m_csMapUserList, TRUE);
	if (m_mapUserList.empty() == true)
		return resultString;

	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.begin();

	bool bFirst = true;
	for(; user_iter != m_mapUserList.end(); ++user_iter)
	{
		CChatUser *pUser = user_iter->second;

		if (pUser->GetID().CompareNoCase(GetMyID()) == 0)
			continue;

		if (bFirst)
		{
			bFirst = false;
		}
		else
		{
			resultString += _T(", ");
		}

		resultString += pUser->GetID();
	}

	return resultString;
}

CString CChatModel::GetAllUserName()
{
	CString resultString(_T(""));

    CSingleLock lock(&m_csMapUserList, TRUE);
    if (m_mapUserList.empty() == true)
		return LoadStringFromResource(IDS_STRING_NO_CONTACT);

	map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.begin();

	bool bFirst = true;
  
	for(; user_iter != m_mapUserList.end(); ++user_iter)
	{
		CChatUser *pUser = user_iter->second;

		if (pUser->GetID().CompareNoCase(GetMyID()) == 0)
			continue;

		if (bFirst)
		{
			bFirst = false;
		}
		else
		{
			resultString += _T(", ");
		}

		resultString += pUser->GetName();
	}

	return resultString;
}

BOOL CChatModel::IsOfflineUser(LPCTSTR strUserID)
{
	BOOL bOffline = TRUE;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::const_iterator iter = m_mapUserList.find(strUserID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		GTUserStatusType userStatusType = pChatUser->GetStatus();

		if(userStatusType == ustOFFLINE)
			bOffline = TRUE;
		else
			bOffline = FALSE;
	}
	else
	{
		bOffline = TRUE;
	}

	return bOffline;
}

CString CChatModel::GetChatUserNickName(LPCTSTR strID)
{
	CString strUserName;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		strUserName = pChatUser->GetNickName();
	}
	else
	{
		strUserName = _T("");
	}

	return strUserName;
}

BOOL CChatModel::IsUserInfoLoaded(LPCTSTR strID)
{
	BOOL bInfoLoaded= FALSE;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		bInfoLoaded = pChatUser->IsUserInfoLoaded();
	}

	return bInfoLoaded;
}

BOOL CChatModel::IsInvitedUser(LPCTSTR strID)
{
    CSingleLock lock(&m_csMapUserList, TRUE);
	//lock.Lock();
	BOOL bInviteStatus = FALSE;
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		bInviteStatus = pChatUser->IsInvited();
	}
	else
	{
		bInviteStatus = FALSE;
	}
	//lock.Unlock();

	return bInviteStatus;
}

BOOL CChatModel::IsOldRoomType()
{
	return (m_strRoomType.Compare(_T("old")) == 0 ? TRUE : FALSE);
		
}

void CChatModel::SetRoomType(LPCTSTR strRoomType)
{
	m_strRoomType = strRoomType;
}

void CChatModel::SetChatMsgTid(UINT tid)
{
	if (tid > m_nMsgTid)
	{
		m_nMsgTid = tid;
	}
}

UINT CChatModel::GetChatMsgTid()
{
	return ++m_nMsgTid;
}

BOOL CChatModel::RemoveChatUserMap(BOOL bAll, LPCTSTR strID)
{
	BOOL res = FALSE;
    CSingleLock lock(&m_csMapUserList, TRUE);
	if(m_mapUserList.empty() == true)
		return res;

	if(bAll == TRUE)
	{
		map<CString, CChatUser*>::iterator user_iter = m_mapUserList.begin();
		for(; user_iter != m_mapUserList.end(); ++user_iter)
		{
			CChatUser* pUser = (CChatUser*)user_iter->second;
			if(pUser)
			{
				TRACE("%s%d %x\n", __FUNCTION__, __LINE__, pUser);
				delete pUser;
				pUser = NULL;
			}
		}

		m_mapUserList.clear();
		m_vectorChatUsr.clear();
	}
	else
	{
		map<CString, CChatUser*>::iterator user_iter = m_mapUserList.find(strID);
		if(user_iter != m_mapUserList.end() )
		{
			TRACE("%s%d %x\n", __FUNCTION__, __LINE__, user_iter->second);		
			delete (user_iter->second);
			m_mapUserList.erase(user_iter);			
			res = TRUE;
		}
		else
		{
			res = FALSE;
		}
	}

	return res;
}

CHAT_MSG* CChatModel::GetFrontChatMessage()
{
	CHAT_MSG* pChatMsg = NULL;

	if (!m_arChatMessage.IsEmpty())
		pChatMsg = (CHAT_MSG*)m_arChatMessage.GetAt(0);

	return pChatMsg;
}

void CChatModel::RemoveFrontChatMessage()
{
	if (!m_arChatMessage.IsEmpty())
	{
		CHAT_MSG* pChatMsg = (CHAT_MSG*)m_arChatMessage.GetAt(0);
		delete pChatMsg;
		m_arChatMessage.RemoveAt(0);
	}
}

void CChatModel::RemoveAllChatMessage()
{
	CHAT_MSG* pItem = NULL;	

	for (int i = 0; i < m_arChatMessage.GetSize(); i++)
	{
		pItem = (CHAT_MSG*)m_arChatMessage.GetAt(i);
		if (pItem)
		{
			delete pItem;
			pItem = NULL;		
		}
	}
	m_arChatMessage.RemoveAll();
}

bool CChatModel::IsExistChatMessage()
{
	if (m_arChatMessage.IsEmpty())
		return false;

	return true;
}

void CChatModel::RefreshTalkName()
{
    CSingleLock lockName(&m_csTalkNameList, TRUE);
    CSingleLock lockUser(&m_csMapUserList, TRUE);
	map<CString, CString>::const_iterator iter = m_mapTalkNameList.begin();
	for(; iter != m_mapTalkNameList.end(); ++iter)
	{
		map<CString, CChatUser*>::const_iterator user_iter = m_mapUserList.find(iter->first);
		if(user_iter != m_mapUserList.end())
		{
			CChatUser* pChatUer = user_iter->second;
			pChatUer->SetNickName(iter->second);
		}
	}
}

BOOL CChatModel::CheckInputActionUserTimeOut(DWORD dwMaxTimeOut)
{
    BOOL bChanged = FALSE;
    if (!m_mapTypingActionUser.empty())
    {
        DWORD dwCurTicket = GetTickCount();
        TypingActionUserContainer::iterator iter = m_mapTypingActionUser.begin();
        while (iter != m_mapTypingActionUser.end())
        {
            if (dwCurTicket >  iter->second + dwMaxTimeOut
                || dwCurTicket < iter->second ) //overflow
            {
                //time out
                m_mapTypingActionUser.erase(iter++);
                bChanged = TRUE;
            }
            else
            {
                ++iter;
            }
        }
    }

    return bChanged;
}

void CChatModel::AddInputActionUser(LPCTSTR strUserName)
{
    DWORD dwCurTicket = GetTickCount();
    m_strLastTypingActionUser = strUserName;
	if(m_mapTypingActionUser.empty() == true)
    {
        m_mapTypingActionUser.insert(make_pair(strUserName, dwCurTicket));
    }
	else
	{
		TypingActionUserContainer::iterator user_iter = m_mapTypingActionUser.find(strUserName);
		if(user_iter == m_mapTypingActionUser.end())
		{
            m_mapTypingActionUser.insert(make_pair(strUserName, dwCurTicket));
		}
        else
        {
            user_iter->second = dwCurTicket;
        }
	}
}

void CChatModel::DelInputActionUser(LPCTSTR strUserName)
{
	if(m_mapTypingActionUser.empty() == true)
    {
		return;
    }
    TypingActionUserContainer::iterator user_iter = m_mapTypingActionUser.find(strUserName);
	if(user_iter != m_mapTypingActionUser.end())
	{
		m_mapTypingActionUser.erase(user_iter);
	}

    if (m_strLastTypingActionUser == strUserName)
    {
        if (m_mapTypingActionUser.empty())
        {
            m_strLastTypingActionUser = TEXT("");
        }
        else
        {
            m_strLastTypingActionUser = m_mapTypingActionUser.begin()->first;
        }
    }
}

CString CChatModel::GetInputActionUserString()
{
	CString strInputActionUser;

	if(m_mapTypingActionUser.empty() == true)
    {
		strInputActionUser =  _T("");
    }
	else
	{	
        ASSERT(!m_strLastTypingActionUser.IsEmpty());
		int nUserSize = m_mapTypingActionUser.size();		
		
		if(nUserSize > 1)
		{
			CString strMsgFormat;
			strMsgFormat.LoadString(IDS_STRING288);
			//vector<CString>::reverse_iterator user_riter = m_mapTypingActionUser.rbegin();
			strInputActionUser.Format(strMsgFormat, m_strLastTypingActionUser, nUserSize - 1);
		}
		else
		{
			CString strAppender;
			strAppender.LoadString(IDS_STRING289);
			//vector<CString>::const_iterator user_iter = m_vecTypingActionUser.begin();
			strInputActionUser = m_strLastTypingActionUser; //(*user_iter);
			strInputActionUser.Append(strAppender);
		}		
	}

	return strInputActionUser;
}

BOOL CChatModel::IsEqualChatDlg(const std::vector<CChatUser *>& userList, LPCTSTR strAddress, int nPort, int nRoomID)
{
	BOOL res = FALSE;
    CSingleLock lock(&m_csMapUserList, TRUE);

	if( (m_strChatSrvAddress.CompareNoCase(strAddress) == 0) &&
		(m_nChatSrvPort == nPort) &&
		(m_nRoomID == nRoomID) )
	{
		std::vector<CChatUser *>::const_iterator iter = userList.begin();
		for(; iter != userList.end(); ++iter)
		{
			CChatUser *pUser = (*iter);
			//Modify by sevin. Because userList contain yourself userID
			if (pUser->GetID() == g_pMessengerActor->GetUserID())
			{
				continue;
			}
			map<CString, CChatUser*>::iterator usermap_iter = m_mapUserList.find(pUser->GetID());
			if(usermap_iter == m_mapUserList.end())
			{
				res = FALSE;
				break;
			}
			else
				res = TRUE;
		}
	}
	else
	{
		res = FALSE;
	}

	return res;

}

BOOL CChatModel::CheckSessionCreateStatus( ESession status )
{
	BOOL result = FALSE;
	if(m_eSessionCreate == status)
		result = TRUE;

	return result;
}

BOOL CChatModel::CheckSessionReason(ESessionReason reason)
{
	BOOL result = FALSE;
	if(m_eSessionReason == reason)
		result = TRUE;

	return result;
}

BOOL CChatModel::CheckinitRoomStatus( EInitRoom status )
{
	BOOL result = FALSE;
	if(m_eInitRoom == status)
		result = TRUE;

	return result;
}

BOOL CChatModel::CheckInviteListStatus( EInviteList status )
{
	BOOL result = FALSE;

	if (m_eInviteList == status)
	{
		result = TRUE;
	}

	return result;
}

void CChatModel::AddMyMessageTid( UINT nTid, LPCTSTR strMessage, CTime tmSend )
{
	CHAT_MYMSG* pMyMsg = new CHAT_MYMSG;
	pMyMsg->nTid = nTid;
	pMyMsg->strMessage = strMessage;
	pMyMsg->tmTime = tmSend;
	m_mapMyTid.insert(make_pair(nTid, pMyMsg));
}

void CChatModel::AddMyTranslateFileInfo( LONG nJobIndex, TranslateFileInfoPtr& pTranslateFileInfo )
{
	FTL::CFAutoLock<CFLockObject>	locker(&m_lockMapTranslateFileJobs);
	m_mapMyTranslateFileJobs.insert(make_pair(nJobIndex, pTranslateFileInfo));
}

void CChatModel::AddMyTranslateTid(UINT nTid, TranslateFileInfoPtr& pTranslateFileInfo)
{
	FTL::CFAutoLock<CFLockObject>	locker(&m_lockMapFileTid);
	m_mapMyFileTid.insert(make_pair(nTid, pTranslateFileInfo));
}

void CChatModel::SetMyMessageTidSendTime(UINT nTid)
{
    if(m_mapMyTid.empty() == false)
    {
        map<UINT, CHAT_MYMSG*>::iterator tid_iter = m_mapMyTid.find(nTid);

        if (tid_iter != m_mapMyTid.end())
        {
            CHAT_MYMSG* pMyMsg = (CHAT_MYMSG*)tid_iter->second;

            CTime time(CTime::GetCurrentTime());
            GetLocaleTime(&time);

            pMyMsg->tmTime = time;
        }
    }
}

void CChatModel::ResetMyMessageTid()
{
	if(m_mapMyTid.empty() == false)
	{
		map< UINT, CHAT_MYMSG* >::iterator myMsg_iter = m_mapMyTid.begin();
		for(; myMsg_iter != m_mapMyTid.end(); ++myMsg_iter)
		{
			CHAT_MYMSG* pMyMsg = myMsg_iter->second;
			delete pMyMsg;
			pMyMsg = NULL;
		}
		m_mapMyTid.clear();
	}
}

void CChatModel::ResetMyTranslateFileInfo()
{
	FTL::CFAutoLock<CFLockObject>	locker(&m_lockMapTranslateFileJobs);
	m_mapMyTranslateFileJobs.clear(); //TranslateFileInfo will delete by SharePtr
}

void CChatModel::ResetMyTranslateTid()
{
	FTL::CFAutoLock<CFLockObject>	locker(&m_lockMapFileTid);
	m_mapMyFileTid.clear();
}

bool CChatModel::AckMyMessageTid( UINT nTid )
{
	if (m_mapMyTid.empty() == true)
		return false;

	map<UINT, CHAT_MYMSG*>::iterator tid_iter = m_mapMyTid.find(nTid);

	if (tid_iter != m_mapMyTid.end())
	{
		CHAT_MYMSG* pMyMsg = (CHAT_MYMSG*)tid_iter->second;
		delete pMyMsg;
		pMyMsg = NULL;

		m_mapMyTid.erase(tid_iter);

		return true;
	}
	else
		return false;
}

bool CChatModel::AckMyTranslateTid( UINT nTid )
{
	FTL::CFAutoLock<CFLockObject>	locker(&m_lockMapFileTid);
	if (m_mapMyFileTid.empty() == true)
	{
		return false;
	}
	map<UINT, TranslateFileInfoPtr>::iterator tid_iter = m_mapMyFileTid.find(nTid);
	if (tid_iter != m_mapMyFileTid.end())
	{
		TranslateFileInfoPtr pTranslateFileInfo = tid_iter->second;
		m_mapMyFileTid.erase(tid_iter);
		return true;
	}
	return false;
}
TranslateFileInfoPtr CChatModel::GetMyTranslateFileInfo(LONG nJobIndex, BOOL bRemove)
{
	FTL::CFAutoLock<CFLockObject>	locker(&m_lockMapTranslateFileJobs);
	TranslateFileInfoPtr pResultTranslateFileInfo(NULL);

	if (m_mapMyTranslateFileJobs.empty() == true)
	{
		return pResultTranslateFileInfo;
	}

	MyTranslateFileJobContainer::iterator iter = m_mapMyTranslateFileJobs.find(nJobIndex);
	if (iter != m_mapMyTranslateFileJobs.end())
	{
		pResultTranslateFileInfo = iter->second;
		if (bRemove)
		{
			m_mapMyTranslateFileJobs.erase(iter);
		}
	}

	return pResultTranslateFileInfo;
}

//void CChatModel::AddRetryTranslateFileInfo(TranslateFileInfoPtr& pTranslateFileInfo)
//{
//	FTL::CFAutoLock<CFLockObject>	locker(&m_lockRetryTranslateFileJobs);
//	m_lstRetryTranslateFileInfo.push_back(pTranslateFileInfo);
//}
//
//TranslateFileInfoPtr CChatModel::GetRetryTranslateFileInfo()
//{
//	FTL::CFAutoLock<CFLockObject>	locker(&m_lockRetryTranslateFileJobs);
//	if (!m_lstRetryTranslateFileInfo.empty())
//	{
//		TranslateFileInfoPtr pFileInfo = m_lstRetryTranslateFileInfo.front();
//		m_lstRetryTranslateFileInfo.pop_front();
//		return pFileInfo;
//	}
//	return TranslateFileInfoPtr(NULL);
//}

void CChatModel::CheckSendMsgAck( vector<CHAT_MYMSG*>& rvec )
{
	if(m_mapMyTid.empty() == true)
		return;

	CTime time(CTime::GetCurrentTime());
	GetLocaleTime(&time);
	map<UINT, CHAT_MYMSG*>::iterator tid_iter = m_mapMyTid.begin();
	for(; tid_iter != m_mapMyTid.end(); ++tid_iter)
	{
		CHAT_MYMSG* pMyMsg = (CHAT_MYMSG*)tid_iter->second;
		CTimeSpan tsDiff = time - pMyMsg->tmTime;
		LONGLONG diffSec = tsDiff.GetTotalSeconds();

		if (diffSec > ACK_DIFFTIME)
		{ 
			rvec.push_back(pMyMsg);
			continue;
		}				
	}
}


void CChatModel::CancelTransferFile(vector<TranslateFileInfoPtr>& rvec)
{
	FTL::CFAutoLock<CFLockObject>	locker(&m_lockMapFileTid);
	if (m_mapMyFileTid.empty())
	{
		return;
	}

	map<UINT, TranslateFileInfoPtr>::iterator iter;
	for (iter = m_mapMyFileTid.begin(); iter != m_mapMyFileTid.end(); ++iter)
	{
		TranslateFileInfoPtr pTranslateFile = (TranslateFileInfoPtr)iter->second;
		rvec.push_back(pTranslateFile);
	}

	m_mapMyFileTid.clear();
}


void CChatModel::CheckSendFileMsgAck(vector<TranslateFileInfoPtr>& rvec)
{
	FTL::CFAutoLock<CFLockObject>	locker(&m_lockMapFileTid);
	if (m_mapMyFileTid.empty())
	{
		return;
	}

	CTime time(CTime::GetCurrentTime());
	GetLocaleTime(&time);

	map<UINT, TranslateFileInfoPtr>::iterator tid_iter = m_mapMyFileTid.begin();
	while (tid_iter != m_mapMyFileTid.end())
	{
		TranslateFileInfoPtr pTranslateFile = (TranslateFileInfoPtr)tid_iter->second;
		CTimeSpan tsDiff = time - pTranslateFile->m_tmSendNotify;
		LONGLONG diffSec = tsDiff.GetTotalSeconds();
		if (diffSec > ACK_DIFFTIME)
		{
			rvec.push_back(pTranslateFile);
			m_mapMyFileTid.erase(tid_iter++);
			continue;
		}
		++tid_iter;
	}
}

GTUserStatusType CChatModel::GetChatUserStatus( LPCTSTR strID )
{
	GTUserStatusType status = ustOFFLINE;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		status = pChatUser->GetStatus();		
	}	

	return status;
}

NTIPTStatusType CChatModel::GetChatUserIptStatus( LPCTSTR strID )
{
	NTIPTStatusType status = iptstNA;
    CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.find(strID);
	if(iter != m_mapUserList.end())
	{
		CChatUser* pChatUser = iter->second;
		status = pChatUser->GetIptStatus();
	}	

	return status;
}

vector<LPCTSTR> CChatModel::GetNotJoinUserList()
{
	vector<LPCTSTR> vecUserList;
	CSingleLock lock(&m_csMapUserList, TRUE);
	map<CString, CChatUser*>::iterator iter = m_mapUserList.begin();
	for(; iter != m_mapUserList.end(); iter++)
	{
		CChatUser* pChatUser = iter->second;
		if(pChatUser->IsInvited() == FALSE && 
			pChatUser->GetStatus() != ustMOBILE &&
			pChatUser->GetStatus() != ustOFFLINE)
		{
			vecUserList.push_back(pChatUser->GetID());
		}
	}
	
	return vecUserList;
}

void CChatModel::UpdateAllChatItemUserStatus(CString strUserId,GTUserStatusType status)
{
	int nCntItem = GetChatItemCount();
	for (int i = 0; i < nCntItem; i++)
	{
		CChatItem* pChItem = (CChatItem*)GetChatItem(i);
		if(strUserId == pChItem->GetChatItemInfo()->strUserID)
		{
			pChItem->UpdateUserStatus(status);
		}
	}
}
	
void CChatModel::UpdateMsgStatus(CString strUserId,UINT nTid, int status)
{
	int nIndex = GetChatItemIndex(strUserId,nTid);
	if (-1 != nIndex)
	{
		CChatItem *pChatItem = GetChatItem(nIndex);
		pChatItem->SetMsgStatus(status);
	}

}

int CChatModel::GetChatItemIndex(const CString &strUserId,int nTid)
{
	int nIndex = -1;
	CSingleLock l(&m_csChatItem, TRUE);
	int nCount = m_ptrArChatItem.GetCount();
	CChatItem* pChItem = NULL;

	//for (int i = 0; i < nCount; i++)
	for (int i = nCount - 1; i >= 0; i--)
	{
		pChItem = (CChatItem*)m_ptrArChatItem.GetAt(i);
		if (pChItem)
		{
			LPChatItemInfo pcii = pChItem->GetChatItemInfo();
			if (pcii
				&& strUserId.CompareNoCase(pcii->strUserID) == 0 
				&& pcii->nTid == nTid)
			{
				nIndex = i;
				break;
			}
		}
	}

	return nIndex;
}
BOOL CChatModel::IsExistChatUserItem()
{
	CSingleLock l(&m_csChatItem, TRUE);
	int nCount = m_ptrArChatItem.GetCount();
	CChatItem* pChItem = NULL;

	for (int i = 0; i < nCount; i++)
	{
		pChItem = (CChatItem*)m_ptrArChatItem.GetAt(i);
		if (!pChItem)
			continue;

		//if(pChItem->eMsgGB == CHAT_MSG_MINE || pChItem->eMsgGB == CHAT_MSG_OTHER)
		//	return TRUE;

		if (pChItem->IsTalkType())
		{
			return TRUE;
		}
	}

	return FALSE;
}

void CChatModel::SetMsgSequence(CString strUserID, UINT nTid, UINT nMsgSeq)
{
	CSingleLock l(&m_csChatItem, TRUE);
	CChatItem* pChItem = NULL;

	int nCntItem = m_ptrArChatItem.GetCount();

	for (int i = nCntItem - 1; i >= 0; i--)
	{
		pChItem = (CChatItem*)m_ptrArChatItem.GetAt(i);
		if (!pChItem)
			continue;

		//if (pChItem->eMsgGB != CHAT_MSG_MINE || pChItem->strUserID.CompareNoCase(strUserID) != 0)
		//	continue;
        if (pChItem->GetTid() == nTid)
        {
            pChItem->SetMsgSeq(nMsgSeq);
        }
		if (pChItem->IsSameUser(strUserID))
		{
			continue;
		}
		if (pChItem->GetTid() == nTid)
		{
			AddMsgSequence(nMsgSeq);
			break;
		}
	}
}

void CChatModel::IncreaseFailCount(CString strUserId,UINT nTid)
{
	CSingleLock l(&m_csChatItem, TRUE);
	CChatItem* pChItem = NULL;

	int nCntItem = m_ptrArChatItem.GetCount();

	for (int i = nCntItem - 1; i >= 0; i--)
	{
		pChItem = (CChatItem*)m_ptrArChatItem.GetAt(i);
		if (!pChItem)
			continue;

		//if (pChItem->eMsgGB != CHAT_MSG_MINE)
		//	continue;

		if (pChItem->IsTalkType() && pChItem->IsSameUser(GetMyID()))
		{
			continue;
		}

		if (pChItem->GetTid() == nTid && pChItem->GetChatItemInfo()->strUserID.CompareNoCase(strUserId) == 0)
		{
			pChItem->IncreaseFailCount();

			break;
		}
	}
}
BOOL CChatModel::RemoveChatItem(int nIndex,BOOL bDelete)
{
	BOOL bResult = FALSE;
	CSingleLock l(&m_csChatItem, TRUE);
	if (nIndex >=0 && nIndex < m_ptrArChatItem.GetCount())
	{
		CChatItem *pChatItem = (CChatItem*)m_ptrArChatItem.GetAt(nIndex);
		m_ptrArChatItem.RemoveAt(nIndex);
		if (bDelete)
		{
			delete pChatItem;
			pChatItem = NULL;
		}

		bResult = TRUE;
	}
	
	return bResult;
}

void CChatModel::ClearChatItemAll()
{
	CSingleLock l(&m_csChatItem, TRUE);
	CChatItem* pChItem = NULL;

	int nCntItem = m_ptrArChatItem.GetCount();

	for (int i = nCntItem - 1; i >= 0; i--)
	{
		pChItem = (CChatItem*)m_ptrArChatItem.GetAt(i);

		if (!pChItem)
			continue;

		SAFE_DELETE(pChItem);
	}

	m_ptrArChatItem.RemoveAll();
}

BOOL CChatModel::IsHaveMessageRecord()
{
	return m_bHaveMsgRecord;
}

int CChatModel::AddChatItemToArray( CChatItem* pChatItem )
{
	m_bHaveMsgRecord = TRUE;
	CSingleLock l(&m_csChatItem, TRUE);
	return m_ptrArChatItem.Add(pChatItem);
}

void CChatModel::AddRecentChatItemToArray( CChatItem* pChatItem )
{
	CSingleLock l(&m_csChatItem, TRUE);
	return m_ptrArChatItem.InsertAt(0, pChatItem);
}

UINT CChatModel::GetMsgCount()
{
	return m_mapMyTid.size();
}
